##################################################
## 初期設定
##################################################
## 「bpp_stats_reader.R」で作成したファイルの名前の設定
bpp_stats.file <- "bpp_stats1.txt"

## チーム成績データをセーブするファイルの名前の設定
team_file.name <- "bpp_teamdata1.txt"

## 個人成績データをセーブするファイルの名前の設定
#### 「bpp_stats_reader.R」の個人成績データを記録するかどうかの選択で、
#### 個人成績データは記録しない設定にした場合は、個人成績データを作成しません。
person_file.name <- "bpp_personaldata1.txt"

##################################################
## 「bpp_stats_reader.R」で作成したデータの読み込み
##################################################
bpp_stats <- read.table(bpp_stats.file, header = TRUE, fileEncoding="cp932", encoding="cp932")

##################################################
## 個人成績とチーム成績の分離
##################################################
## 個人成績データの抽出
person_stats <- bpp_stats[bpp_stats$team == 0, ]
person.flag <- substring(as.character(unique(person_stats$name)), 1, 1)

## チーム成績データの抽出
team_stats <- bpp_stats[bpp_stats$team != 0, ]

##################################################
## 新しい列名での個人成績データの作成
##################################################
## 野手の場合
if (person.flag == "b") {
  ## 各列ベクトルの作成
  ## チーム
  ID <- person_stats$team
  name <- person_stats$name
  ## 試合成績
  W <- NA
  L <- NA
  D <- NA
  WP <- NA
  AVG <- person_stats$b.hits / person_stats$b.ab
  ERA <- NA
  ## 打撃成績
  G <- person_stats$b.games
  AB <- person_stats$b.ab
  H <- person_stats$b.hits
  HR <- person_stats$b.hr
  RBI <- person_stats$b.rbi
  SB <- person_stats$b.sb
  BB <- person_stats$b.bb
  SO <- person_stats$b.so
  ## 投手成績
  GP <- NA
  IP <- NA
  CG <- NA
  S <- NA
  SP <- NA
  Ha <- NA
  HRa <- NA
  BBa <- NA
  K <- NA
  ER <- NA
  ## 守備成績
  E <- person_stats$b.errors
  ## 付加情報
  option <- person_stats$option
}

## 投手の場合
if (person.flag == "p") {
  ## 各列ベクトルの作成
  ## チーム
  ID <- person_stats$team
  name <- person_stats$name
  ## 試合成績
  W <- person_stats$p.wins
  L <- person_stats$p.losses
  D <- NA
  WP <- NA
  AVG <- NA
  ERA <- 9 * person_stats$p.er / person_stats$p.ip
  ## 打撃成績
  G <- NA
  AB <- NA
  H <- NA
  HR <- NA
  RBI <- NA
  SB <- NA
  BB <- NA
  SO <- NA
  ## 投手成績
  GP <- person_stats$p.games
  IP <- person_stats$p.ip
  CG <- person_stats$p.cg
  S <- person_stats$p.saves
  SP <- person_stats$p.sp
  Ha <- person_stats$p.hits
  HRa <- person_stats$p.hr
  BBa <- person_stats$p.bb
  K <- person_stats$p.so
  ER <- person_stats$p.er
  ## 守備成績
  E <- NA
  ## 付加情報
  option <- person_stats$option
}

## 新しい列名での個人成績データの作成
if ((person.flag == "b") || (person.flag == "p")) {
  bpp_personaldata <- data.frame("ID" = ID, "name" = name,  
                                 "W" = W, "L" = L, "D" = D, "WP" = WP, 
                                 "AVG" = AVG, "ERA" = ERA, 
                                 "G" = G, "AB" = AB, 
                                 "H" = H, "HR" = HR, "RBI" = RBI, "SB" = SB, 
                                 "BB" = BB, "SO" = SO, 
                                 "GP" = GP, "IP" = IP, "CG" = CG,
                                 "S" = S, "SP" = SP, 
                                 "Ha" = Ha, "HRa" = HRa, "BBa" = BBa, "K" = K, 
                                 "ER" = ER, 
                                 "E" = E, 
                                 "option" = option)
}

##################################################
## 新しい列名での個人成績データの書き込み
##################################################
if ((person.flag == "b") || (person.flag == "p")) {
  write.table(bpp_personaldata, person_file.name, fileEncoding="cp932")
}

##################################################
## 新しい列名でのチーム成績データの作成
##################################################
## 各列ベクトルの作成
## チーム
ID <- team_stats$team
team <- team_stats$name
## 試合成績
W <- team_stats$p.wins
L <- team_stats$p.losses
D <- team_stats$s.games - (team_stats$p.wins + team_stats$p.losses)
WP <- team_stats$p.win / team_stats$s.games
AVG <- team_stats$b.hits / team_stats$b.ab
ERA <- 9 * team_stats$p.er / team_stats$p.ip
## 打撃成績
G <- team_stats$b.games
AB <- team_stats$b.ab
H <- team_stats$b.hits
HR <- team_stats$b.hr
RBI <- team_stats$b.rbi
SB <- team_stats$b.sb
BB <- team_stats$b.bb
SO <- team_stats$b.so
## 投手成績
GP <- team_stats$p.games
IP <- team_stats$p.ip
CG <- team_stats$p.cg
S <- team_stats$p.saves
SP <- team_stats$p.sp
Ha <- team_stats$p.hits
HRa <- team_stats$p.hr
BBa <- team_stats$p.bb
K <- team_stats$p.so
ER <- team_stats$p.er
## 守備成績
E <- team_stats$b.errors
## 付加情報
option <- team_stats$option

## 新しい列名でのチーム成績データの作成
bpp_teamdata <- data.frame("ID" = ID, "team" = team,  
                           "W" = W, "L" = L, "D" = D, "WP" = WP, 
                           "AVG" = AVG, "ERA" = ERA, 
                           "G" = G, "AB" = AB, 
                           "H" = H, "HR" = HR, "RBI" = RBI, "SB" = SB, 
                           "BB" = BB, "SO" = SO, 
                           "GP" = GP, "IP" = IP, "CG" = CG,
                           "S" = S, "SP" = SP, 
                           "Ha" = Ha, "HRa" = HRa, "BBa" = BBa, "K" = K, 
                           "ER" = ER, 
                           "E" = E, 
                           "option" = option)

##################################################
## 新しい列名でのチーム成績データの書き込み
##################################################
write.table(bpp_teamdata, team_file.name, fileEncoding="cp932")

##################################################
## データ変換完了の表示
##################################################
cat("データ変換完了！\n")

##################################################
## FUJINO, Kazuhiro 2020
## http://fujino.la.coocan.jp/index.htm
##################################################