##################################################
## 初期設定
##################################################
## 「bpp_stats_reader.R」で作成したファイルの名前の設定
bpp_stats.file <- "bpp_stats1.txt"

## 「bpp_converter.R」で作成したファイルの名前の設定
bpp_data.file <- "bpp_teamdata1.txt"

##################################################
## BPP通算成績データの読み込み
##################################################
## 「bpp_stats_reader.R」で作成したデータの読み込み
bpp_stats <- read.table(bpp_stats.file, header = TRUE, fileEncoding="cp932", encoding="cp932")
## 「bpp_converter.R」で作成したデータの読み込み
bpp_data <- read.table(bpp_data.file, header = TRUE, fileEncoding="cp932", encoding="cp932")

##################################################
## 列名の設定（names()関数を使えば良いと思うけど、備忘用）
##################################################
bpp_data.list <- c("ID", "team", "W", "L", "D", 
                   "WP", "AVG", "ERA", 
                   "G", "AB", "H", "HR", "RBI", "SB", 
                   "BB", "SO", 
                   "GP", "IP", "CG", "S", "SP", 
                   "Ha", "HRa", "BBa", "K", "ER", 
                   "E", 
                   "option")

##################################################
## データフレームの内容を、HTMLの表を作成するタグ形式で出力
##################################################
html.table <- function(data.name) {
  library(xtable)
  html.table <- xtable(data.name) 
  return(print(html.table, type = "html", include.rownames = FALSE))
}

##################################################
## 特定の「"option"」のデータのみ抽出
##################################################
option.data <- function(data.name, option.name) {
  option.data <- subset(data.name, data.name$option == option.name)
  invisible(option.data)
}

##################################################
## 「bpp_stats_reader.R」で作成したデータから
## 「season.num」の試行（シーズン）のデータを抽出
##################################################
season.subset <- function(season.num) {
   row.num <- (13 * (season.num - 1) + 1):(13 * (season.num - 1) + 13)
   season.dat <- bpp_stats[row.num, ]
   invisible(season.dat)
}

##################################################
## 「bpp_stats_reader.R」で作成したデータから
## 「season.num」の試行（シーズン）のデータを削除
##################################################
season.delete <- function(season.num) {
   row.num <- 1:nrow(bpp_stats)
   delete.row.num <- (13 * (season.num - 1) + 1):(13 * (season.num - 1) + 13)
   row.num <- row.num[-1 * delete.row.num]
   season.dat <- bpp_stats[row.num, ]
   invisible(season.dat)
}

##################################################
## 「bpp_converter.R」で作成したデータから
## 「"option"」条件ごとに標本平均のデータを作成
##################################################
bpp_data.mean <- function() {

  ## チーム名とオプション名のリストの作成
  team.list <- unique(bpp_data$team)
  option.list <- unique(bpp_data$option)

  ## オプション条件ごとに標本平均を作成
  bpp_mean <- data.frame(NULL)
  for (i in option.list) {
    ## オプション条件に合うデータを抽出
    i_option.stats <- subset(bpp_data, bpp_data$option == i)
    ## 各チームごとに標本平均を計算
    j_option.mean <- data.frame(NULL)
    for (j in 1:12) {
      ## チームごとにデータを抽出
      j_team.stats <- subset(i_option.stats, i_option.stats$ID == j)
      ## 標本平均を計算する列の抽出
      j_team.stats <- j_team.stats[3:27]
      ## 標本平均の計算
      j_team.mean <- apply(j_team.stats, MARGIN = 2, mean)
      j_team.mean <- matrix(j_team.mean, nrow = 1, ncol = 25)
      j_team.mean <- data.frame(j_team.mean)
      ## 付加情報の追加
      j_team.mean <- cbind(j, team.list[j], j_team.mean, i)
      ## 標本平均データの結合
      j_option.mean <- rbind(j_option.mean, j_team.mean)
    }
    ## オプション条件ごとの標本平均データの結合
    bpp_mean <- rbind(bpp_mean, j_option.mean)
  }

  ## 列名の付加
  names(bpp_mean) <- bpp_data.list
  ## 出力結果
  invisible(bpp_mean)
}

##################################################
## 「bpp_converter.R」で作成したデータから
## 「"option"」条件ごとに標準誤差のデータを作成
##################################################
bpp_data.sd <- function() {

  ## チーム名とオプション名のリストの作成
  team.list <- unique(bpp_data$team)
  option.list <- unique(bpp_data$option)

  ## オプション条件ごとに標準誤差を作成
  bpp_sd <- data.frame(NULL)
  for (i in option.list) {
    ## オプション条件に合うデータを抽出
    i_option.stats <- subset(bpp_data, bpp_data$option == i)
    ## 各チームごとに標準誤差を計算
    j_option.sd <- data.frame(NULL)
    for (j in 1:12) {
      ## チームごとにデータを抽出
      j_team.stats <- subset(i_option.stats, i_option.stats$ID == j)
      ## 標準誤差を計算する列の抽出
      j_team.stats <- j_team.stats[3:27]
      ## 標準誤差の計算
      j_team.sd <- apply(j_team.stats, MARGIN = 2, sd)
      j_team.sd <- matrix(j_team.sd, nrow = 1, ncol = 25)
      j_team.sd <- data.frame(j_team.sd)
      ## 付加情報の追加
      j_team.sd <- cbind(j, team.list[j], j_team.sd, i)
      ## 標準誤差データの結合
      j_option.sd <- rbind(j_option.sd, j_team.sd)
    }
    ## オプション条件ごとの標準誤差データの結合
    bpp_sd <- rbind(bpp_sd, j_option.sd)
  }

  ## 列名の付加
  names(bpp_sd) <- bpp_data.list
  ## 出力結果
  invisible(bpp_sd)
}

##################################################
## 「bpp_converter.R」で作成したデータを用いて
## 2つの「"option"」のデータの標本平均の差を計算
##################################################
dif.mean <- function(option1, option2) {

 ## チーム名のリストの作成
  team.list <- unique(bpp_data$team)

  ## オプション条件に合うデータを抽出
  bpp_data1 <- subset(bpp_data, bpp_data$option == option1)
  bpp_data2 <- subset(bpp_data, bpp_data$option == option2)

  ## 各チームごとに標本平均の差を計算
  dif.mean <- data.frame(NULL)
  for (i in 1:12) {
    ## チームごとにデータを抽出
    i_team.stats1 <- subset(bpp_data1, bpp_data1$ID == i)
    i_team.stats2 <- subset(bpp_data2, bpp_data1$ID == i)
    ## 標本平均を計算する列の抽出
    i_team.stats1 <- i_team.stats1[3:27]
    i_team.stats2 <- i_team.stats2[3:27]
    ## 標本平均の計算
    i_team.mean1 <- apply(i_team.stats1, MARGIN = 2, mean)
    i_team.mean2 <- apply(i_team.stats2, MARGIN = 2, mean)
    ## 標本平均の差の計算
    i_team.mean <- i_team.mean1 - i_team.mean2
    i_team.mean <- matrix(i_team.mean, nrow = 1, ncol = 25)
    i_team.mean <- data.frame(i_team.mean)
    ## 付加情報の追加
    i_team.mean <- cbind(i, team.list[i], i_team.mean)
    ## 各チームごとの標本平均の差データの結合
    dif.mean <- rbind(dif.mean, i_team.mean)
  }

  ## 列名の付加
  names(dif.mean) <- bpp_data.list[1:27]
  ## 出力結果
  invisible(dif.mean)
}

##################################################
## 「bpp_converter.R」で作成したデータを用いて
## 2つの「"option"」のデータの標本平均の差の標準誤差を計算
##################################################
dif.mean.sd <- function(option1, option2) {

 ## チーム名のリストの作成
  team.list <- unique(bpp_data$team)

  ## オプション条件に合うデータを抽出
  bpp_data1 <- subset(bpp_data, bpp_data$option == option1)
  bpp_data2 <- subset(bpp_data, bpp_data$option == option2)

  ## 各チームごとに標本平均の差の標準誤差を計算
  dif.mean.sd <- data.frame(NULL)
  for (i in 1:12) {
    ## チームごとにデータを抽出
    i_team.stats1 <- subset(bpp_data1, bpp_data1$ID == i)
    i_team.stats2 <- subset(bpp_data2, bpp_data1$ID == i)
    ## 計算する列の抽出
    i_team.stats1 <- i_team.stats1[3:27]
    i_team.stats2 <- i_team.stats2[3:27]
    ## 標本平均の差の分散の計算
    i_team.var1 <- apply(i_team.stats1, MARGIN = 2, var)
    i_team.var2 <- apply(i_team.stats2, MARGIN = 2, var)
    ## 標本サイズの計算
    i_team.n1 <- apply(i_team.stats1, MARGIN = 2, length)
    i_team.n2 <- apply(i_team.stats2, MARGIN = 2, length)
    ## 標本平均の差の標準誤差の計算
    i_team.sd <- sqrt((i_team.var1 / i_team.n1) + (i_team.var2 / i_team.n2))
    i_team.sd <- matrix(i_team.sd, nrow = 1, ncol = 25)
    i_team.sd <- data.frame(i_team.sd)
    ## 付加情報の追加
    i_team.sd <- cbind(i, team.list[i], i_team.sd)
    ## 各チームごとの標本平均の差データの結合
    dif.mean.sd <- rbind(dif.mean.sd, i_team.sd)
  }

  ## 列名の付加
  names(dif.mean.sd) <- bpp_data.list[1:27]
  ## 出力結果
  invisible(dif.mean.sd)
}

##################################################
## 「bpp_converter.R」で作成したデータを用いて
## 2つの「"option"」のデータの標本平均の差（％表記）を計算
##################################################
dif.mean.pct <- function(option1, option2) {

 ## チーム名のリストの作成
  team.list <- unique(bpp_data$team)

  ## オプション条件に合うデータを抽出
  bpp_data1 <- subset(bpp_data, bpp_data$option == option1)
  bpp_data2 <- subset(bpp_data, bpp_data$option == option2)

  ## 各チームごとに標本平均の差を計算
  dif.mean.pct <- data.frame(NULL)
  for (i in 1:12) {
    ## チームごとにデータを抽出
    i_team.stats1 <- subset(bpp_data1, bpp_data1$ID == i)
    i_team.stats2 <- subset(bpp_data2, bpp_data1$ID == i)
    ## 標本平均を計算する列の抽出
    i_team.stats1 <- i_team.stats1[3:27]
    i_team.stats2 <- i_team.stats2[3:27]
    ## 標本平均の計算
    i_team.mean1 <- apply(i_team.stats1, MARGIN = 2, mean)
    i_team.mean2 <- apply(i_team.stats2, MARGIN = 2, mean)
    ## 標本平均の差（％表記）の計算
    i_team.mean.pct <- 100 * ((i_team.mean1 - i_team.mean2) / i_team.mean2)
    i_team.mean.pct <- matrix(i_team.mean.pct, nrow = 1, ncol = 25)
    i_team.mean.pct <- data.frame(i_team.mean.pct)
    ## 付加情報の追加
    i_team.mean.pct <- cbind(i, team.list[i], i_team.mean.pct)
    ## 各チームごとの標本平均の差データの結合
    dif.mean.pct <- rbind(dif.mean.pct, i_team.mean.pct)
  }

  ## 列名の付加
  names(dif.mean.pct) <- bpp_data.list[1:27]
  ## 出力結果
  invisible(dif.mean.pct)
}

##################################################
## 「bpp_converter.R」で作成したデータを用いて
## 2つの「"option"」のデータの差について
## 差がゼロという帰無仮説に対するp値を計算
##################################################
bpp_p.value <- function(option1, option2) {

  ## チーム名のリストの作成
  team.list <- unique(bpp_data$team)

  ## オプション条件に合うデータを抽出
  bpp_data1 <- subset(bpp_data, bpp_data$option == option1)
  bpp_data2 <- subset(bpp_data, bpp_data$option == option2)

  ## チームごとにp値を計算
  p.table <- NULL
  for (i in 1:12) {
    ## チームごとにデータを抽出
    team.data1 <- subset(bpp_data1, bpp_data1$ID == i)
    team.data2 <- subset(bpp_data2, bpp_data2$ID == i)
    ## 列ごとにp値を計算
    p.vector <- NULL
    for (j in 3:27) {
      test.result <- t.test(team.data1[j], team.data2[j], var.equal = FALSE)
      p.value <- test.result$p.value
      p.vector <- c(p.vector, p.value) 
    }
    ## 型変換
    p.matrix <- matrix(p.vector, nrow = 1, ncol = 25)
    p.dataframe <- data.frame(p.matrix)
    ## 付加情報の追加
    p.dataframe <- cbind(i, team.list[i], p.dataframe)
    ## オプション条件ごとのp値データの結合
    p.table <- rbind(p.table, p.dataframe)
  }

  ## 列名の付加
  names(p.table) <- bpp_data.list[1:27]
  ## 出力結果
  invisible(p.table)
}

##################################################
## 「bpp_converter.R」で作成したデータを用いて
## 2つの「"option"」のデータの差について有意差の有無を判定
##################################################
bpp_sig <- function(option1, option2, alpha = 0.05) {

  ## チーム名のリストの作成
  team.list <- unique(bpp_data$team)

  ## オプション条件に合うデータを抽出
  bpp_data1 <- subset(bpp_data, bpp_data$option == option1)
  bpp_data2 <- subset(bpp_data, bpp_data$option == option2)

  ## チームごとにp値を計算
  sig.table <- NULL
  for (i in 1:12) {
    ## チームごとにデータを抽出
    team.data1 <- subset(bpp_data1, bpp_data1$ID == i)
    team.data2 <- subset(bpp_data2, bpp_data2$ID == i)
    ## 列ごとにp値を計算し、有意差の有無を判定
    sig.vector <- NULL
    for (j in 3:27) {
      test.result <- t.test(team.data1[j], team.data2[j], var.equal = FALSE)
      p.value <- test.result$p.value
      ## 有意差の有無を判定
      sig.value <- p.value < alpha
      sig.vector <- c(sig.vector, sig.value) 
    }
    ## 型変換
    sig.matrix <- matrix(sig.vector, nrow = 1, ncol = 25)
    sig.dataframe <- data.frame(sig.matrix)
    ## 付加情報の追加
    sig.dataframe <- cbind(i, team.list[i], sig.dataframe)
    ## オプション条件ごとの有意差データの結合
    sig.table <- rbind(sig.table, sig.dataframe)
  }

  ## 列名の付加
  names(sig.table) <- bpp_data.list[1:27]
  ## 出力結果
  invisible(sig.table)
}

##################################################
## 「bpp_converter.R」で作成したデータを用いて
## 特定の「"列名"」「ID」「"option"」でヒストグラムを作成
##################################################
bpp_hist <- function(stats.name, team.id, option.name = "ALL") {

  ## チーム名のリストの作成
  team.list <- unique(bpp_data$team)

  ## オプション条件に合うデータを抽出
  if (option.name != "ALL") {
    bpp_data.a <- subset(bpp_data, bpp_data$option == option.name)
  } else {
    bpp_data.a <- bpp_data
  }
  ## チーム条件に合うデータを抽出
  bpp_data.b <- subset(bpp_data.a, bpp_data.a$ID == team.id)
  ## 列名条件に合うデータを抽出
  bpp_data.c <- bpp_data.b[[stats.name]]

  ## ヒストグラム作成のためのパラメータ設定
  bpp_min <- min(bpp_data.c)
  bpp_max <- max(bpp_data.c)
  hist.main <- paste("team: ", team.list[team.id], 
                     " (option: ", option.name, 
                     ", n: ", length(bpp_data.c), ")", sep = "")
  ## ヒストグラムの作成（複数のヒストグラムを並べるために「main」引数をNULLとする）
  par(mex = 1, mar = c(3.5, 3, 2, 2))
    ## mex: 余白などを指定する際の数値の1単位の大きさ
    ## mar: 底辺，左側，上側，右側の余白の大きさ
  hist(bpp_data.c, freq = FALSE, col = "#cccccc", 
       breaks = seq(bpp_min, bpp_max, (bpp_max - bpp_min) / 20), 
       main = hist.main, xlab = stats.name, ylab = "相対尤度（確率密度）", 
       mgp = c(1.6, 0.7, 0), cex = 1, cex.axis = 1, cex.lab = 1, tcl = -0.5)
    ## mgp: 軸タイトル，軸ラベル，軸線の描かれる位置
    ## cex: 文字の大きさ
    ## cex.axis: 軸数値の文字の大きさ
    ## cex.lab: 軸ラベルの文字の大きさ
    ## tcl: 軸の目盛線の長さ（＋は内側，－は外側に目盛を入れる）

  ## 正規分布の描画
  bpp_mean <- mean(bpp_data.c)
  bpp_sd <- sd(bpp_data.c)
  bpp_by <- (max(bpp_data.c) - min(bpp_data.c)) / 100
  x <- seq(from = bpp_min, to = bpp_max, by = bpp_by)
  lines(x, dnorm(x, mean = bpp_mean, sd = bpp_sd))
}

##################################################
## 「bpp_converter.R」で作成したデータを用いて
## 2つの異なる「"option"」のヒストグラムを重ねて表示
##################################################
bpp_hist.2 <- function(stats.name, team.id, option.name1, option.name2 = "ALL") {

  ## チーム名のリストの作成
  team.list <- unique(bpp_data$team)

  ## チーム条件に合うデータを抽出
  bpp_data.team <- subset(bpp_data, bpp_data$ID == team.id)
  ## オプション条件に合うデータを抽出
  bpp_data.1 <- subset(bpp_data.team, bpp_data.team$option == option.name1)
   bpp_data.1 <- bpp_data.1[[stats.name]]
  if (option.name2 != "ALL") {
    bpp_data.2 <- subset(bpp_data.team, bpp_data.team$option == option.name2)
     bpp_data.2 <- bpp_data.2[[stats.name]]
  } else {
    bpp_data.2 <- bpp_data.team[[stats.name]]
  }
  bpp_data.3 <- c(bpp_data.1, bpp_data.2)

  ## x軸とy軸の範囲を設定
  x.min <- min(bpp_data.3)
  x.max <- max(bpp_data.3)
  y.min <- 0
  y.max <- dnorm(mean(bpp_data.3), mean = mean(bpp_data.3), sd = sd(bpp_data.3))
   y.max <- 2 * y.max
  ## 色の設定
  color.1 <- rgb(red = 1, green = 0, blue = 0, alpha = 0.5)
  color.2 <- rgb(red = 0, green = 0, blue = 1, alpha = 0.5)
  ## タイトルの設定
  hist.main <- paste("team:", team.list[team.id])
  legend1 <- paste("【赤】", option.name1, " (n: ", length(bpp_data.1), ")", sep = "")
  legend2 <- paste("【青】", option.name2, " (n: ", length(bpp_data.2), ")", sep = "")

  ## ヒストグラムの作成・その1
  par(mex = 1, mar = c(3.5, 3, 2, 3))
    ## mex: 余白などを指定する際の数値の1単位の大きさ
    ## mar: 底辺，左側，上側，右側の余白の大きさ
  hist(bpp_data.1, freq = FALSE, col = color.1, 
       breaks = seq(x.min, x.max, (x.max - x.min) / 20), 
       ylim = c(y.min, y.max), 
       main = hist.main, xlab = stats.name, ylab = "相対尤度（確率密度）", 
       mgp = c(1.6, 0.7, 0), cex = 1, cex.axis = 1, cex.lab = 1, tcl = -0.5)
    ## mgp: 軸タイトル，軸ラベル，軸線の描かれる位置
    ## cex: 文字の大きさ
    ## cex.axis: 軸数値の文字の大きさ
    ## cex.lab: 軸ラベルの文字の大きさ
    ## tcl: 軸の目盛線の長さ（＋は内側，－は外側に目盛を入れる）

  ## ヒストグラムの作成・その2（重複するので縦軸・横軸は描画しない）
  par(new = TRUE)
  hist(bpp_data.2, freq = FALSE, col = color.2, 
       breaks = seq(x.min, x.max, (x.max - x.min) / 20), 
       ylim = c(y.min, y.max), 
       axes = FALSE, ann = FALSE)

  ## 凡例を作成
  legend("topright", c(legend1, legend2), 
         col = 1:2, text.col = c("red", "blue"), bty = "n", 
         cex = 0.8)
    ## cex: 文字の大きさ

  ## 標本平均を描画
  abline(v = mean(bpp_data.1), col = "red")
  abline(v = mean(bpp_data.2), col = "blue")

  ## 正規分布の描画
  bpp_by <- (x.max - x.min) / 100
  x <- seq(from = x.min, to = x.max, by = bpp_by)
  lines(x, dnorm(x, mean = mean(bpp_data.1), sd = sd(bpp_data.1)), 
        col = "red")
  lines(x, dnorm(x, mean = mean(bpp_data.2), sd = sd(bpp_data.2)), 
        col = "blue")
}

##################################################
## 「真の平均値」への収束の可視化
##################################################
mean.plot <- function(stats.name, team.id, option.name = "ALL") {

  ## チーム名のリストの作成
  team.list <- unique(bpp_data$team)

  ## オプション条件に合うデータを抽出
  if (option.name != "ALL") {
    bpp_data.a <- subset(bpp_data, bpp_data$option == option.name)
  } else {
    bpp_data.a <- bpp_data
  }
  ## チーム条件に合うデータを抽出
  bpp_data.b <- subset(bpp_data.a, bpp_data.a$ID == team.id)
  ## 列名条件に合うデータを抽出
  bpp_data.c <- bpp_data.b[[stats.name]]

  ## 試行数ごとに標本平均の算出
  stats.mean <- rep(NULL, length(bpp_data.c))
  for (i in 1:length(bpp_data.c)) {
    stats.mean[i] <- mean(bpp_data.c[1:i])
  }

  ## グラフの作成
  y.label <- paste(stats.name, "の標本平均",  sep = " ")
  plot.main <- paste("team: ", team.list[team.id], 
                     " (option: ", option.name, 
                     ", n: ", length(bpp_data.c), ")", sep = "")
  x <- 1:length(bpp_data.c)
  plot(x, stats.mean, type = "l", lwd = 2, mgp = c(2, 1, 0), 
       main = plot.main, xlab = "試行数", ylab = y.label)
  ## 「真の平均値」の描写
  abline(h = stats.mean[length(bpp_data.c)], col = "blue")
}

##################################################
## 「真の平均値」への収束の可視化（ヒストグラムとの合成用）
##################################################
mean.plot_hist <- function(stats.name, team.id, option.name = "ALL") {

  ## ヒストグラムの作成
  bpp_hist(stats.name, team.id, option.name)

  ## チーム名のリストの作成
  team.list <- unique(bpp_data$team)

  ## オプション条件に合うデータを抽出
  if (option.name != "ALL") {
    bpp_data.a <- subset(bpp_data, bpp_data$option == option.name)
  } else {
    bpp_data.a <- bpp_data
  }
  ## チーム条件に合うデータを抽出
  bpp_data.b <- subset(bpp_data.a, bpp_data.a$ID == team.id)
  ## 列名条件に合うデータを抽出
  bpp_data.c <- bpp_data.b[[stats.name]]

  ## 試行数ごとに標本平均の算出
  stats.mean <- rep(NULL, length(bpp_data.c))
  for (i in 1:length(bpp_data.c)) {
    stats.mean[i] <- mean(bpp_data.c[1:i])
  }

  ## グラフ作成のためのパラメータ設定
  plot.main <- paste("team: ", team.list[team.id], 
                     " (option: ", option.name, 
                     ", n: ", length(bpp_data.c), ")", sep = "")
  bpp_min <- min(bpp_data.c)
  bpp_max <- max(bpp_data.c)
  x <- 1:length(bpp_data.c)
  ## グラフの作成
  par(new = TRUE, mex = 1, mar = c(3.5, 3, 2, 2))
    ## mex: 余白などを指定する際の数値の1単位の大きさ
    ## mar: 底辺，左側，上側，右側の余白の大きさ
  plot(stats.mean, x, type = "l", lwd = 2, col = "red", 
       xlim = c(bpp_min, bpp_max), 
       main = plot.main, xlab = "", ylab = "", 
       mgp = c(1, 0, 0), xaxt  = "n", yaxt = "n", cex.axis = 0.8, tcl = 0.3)
    ## mgp: 軸タイトル，軸ラベル，軸線の描かれる位置
    ## xaxt, yaxt: 軸の描写の有無（重複するので縦軸・横軸は描画しない）
    ## cex.axis: 軸数値の文字の大きさ
    ## tcl: 軸の目盛線の長さ（＋は内側，－は外側に目盛を入れる）
  ## 右側の縦軸の描画
  axis(side = 4, col.axis = "red", 
       mgp = c(0, 0.7, 0), cex.axis = 1, tcl = -0.5)
    ## mgp: 軸タイトル，軸ラベル，軸線の描かれる位置
    ## cex.axis: 軸数値の文字の大きさ
    ## tcl: 軸の目盛線の長さ（＋は内側，－は外側に目盛を入れる）
  text.x <- bpp_max - ((bpp_max - bpp_min) / 500)
    ## /の後ろの数字は、軸ラベルが右側の縦軸から左方向にどれだけ離すかを示す（通常は1000くらい？）
  text.y <- length(bpp_data.c) / 2
  text("試行数", x = text.x, y = text.y, col = "red", srt = 90, cex = 1) 
    ## cex: 文字の大きさ

  ## プラスマイナス1標準誤差の範囲の描写
  line.sd <- sd(bpp_data.c)
  line.mean <- mean(bpp_data.c)
  abline(v = line.mean + line.sd, col = "blue")
  abline(v = line.mean - line.sd, col = "blue")
}

##################################################
## 「bpp_converter.R」で作成したデータを用いて
## 2つの異なる「"option"」のヒストグラムを重ねて表示
## さらに、「真の平均値」への収束の可視化を重ねて表示
##################################################
mean.plot_hist.2 <- function(stats.name, team.id, option.name1, option.name2 = "ALL") {

  ## ヒストグラムの作成
  bpp_hist.2(stats.name, team.id, option.name1, option.name2)

  ## チーム名のリストの作成
  team.list <- unique(bpp_data$team)

  ## チーム条件に合うデータを抽出
  bpp_data.team <- subset(bpp_data, bpp_data$ID == team.id)
  ## オプション条件に合うデータを抽出
  bpp_data.1 <- subset(bpp_data.team, bpp_data.team$option == option.name1)
   bpp_data.1 <- bpp_data.1[[stats.name]]
  if (option.name2 != "ALL") {
    bpp_data.2 <- subset(bpp_data.team, bpp_data.team$option == option.name2)
     bpp_data.2 <- bpp_data.2[[stats.name]]
  } else {
    bpp_data.2 <- bpp_data.team[[stats.name]]
  }
  bpp_data.3 <- c(bpp_data.1, bpp_data.2)

  ## x軸の範囲を設定
  x.min <- min(bpp_data.3)
  x.max <- max(bpp_data.3)

  ## 試行数ごとに標本平均の算出（その1）
  stats.mean.1 <- rep(NULL, length(bpp_data.1))
  for (i in 1:length(bpp_data.1)) {
    stats.mean.1[i] <- mean(bpp_data.1[1:i])
  }
  ## グラフの作成（その1）
  x.1 <- 1:length(bpp_data.1)
  par(new = TRUE, mex = 1, mar = c(3.5, 3, 2, 3))
    ## mex: 余白などを指定する際の数値の1単位の大きさ
    ## mar: 底辺，左側，上側，右側の余白の大きさ
  plot(stats.mean.1, x.1, type = "l", lwd = 2, col = "red", 
       xlim = c(x.min, x.max), 
       main = "", xlab = "", ylab = "", 
　　　　　　　mgp = c(2, 1, 0), xaxt  = "n", yaxt = "n", cex.axis = 1, tcl = 0.5)
    ## mgp: 軸タイトル，軸ラベル，軸線の描かれる位置
    ## xaxt, yaxt: 軸の描写の有無（重複するので縦軸・横軸は描画しない）
    ## cex.axis: 軸数値の文字の大きさ
    ## tcl: 軸の目盛線の長さ（＋は内側，－は外側に目盛を入れる）
  ## 右側の縦軸の描画（その1）
  axis(side = 4, col.axis = "red", 
       mgp = c(0, 0.7, 0), cex.axis = 1, tcl = -0.5)
    ## mgp: 軸タイトル，軸ラベル，軸線の描かれる位置
    ## cex.axis: 軸数値の文字の大きさ
    ## tcl: 軸の目盛線の長さ（＋は内側，－は外側に目盛を入れる）

  ## 試行数ごとに標本平均の算出（その2）
  stats.mean.2 <- rep(NULL, length(bpp_data.2))
  for (i in 1:length(bpp_data.2)) {
    stats.mean.2[i] <- mean(bpp_data.2[1:i])
  }
  ## グラフの作成（その2）
  x.2 <- 1:length(bpp_data.2)
  par(new = TRUE)
  plot(stats.mean.2, x.2, type = "l", lwd = 2, col = "blue", 
       mgp = c(2, 1, 0), xlim = c(x.min, x.max), xaxt  = "n", yaxt = "n", 
       main = "", xlab = "", ylab = "", cex.axis = 1, tcl = 0.5)
    ## mgp: 軸タイトル，軸ラベル，軸線の描かれる位置
    ## xaxt, yaxt: 軸の描写の有無（重複するので縦軸・横軸は描画しない）
    ## cex.axis: 軸数値の文字の大きさ
    ## tcl: 軸の目盛線の長さ（＋は内側，－は外側に目盛を入れる）
  ## 右側の縦軸の描画（その2）
  axis(side = 4, col.axis = "blue", 
       mgp = c(0, 1.6, 0), cex.axis = 1, tcl = -0.5)
    ## mgp: 軸タイトル，軸ラベル，軸線の描かれる位置
    ## cex.axis: 軸数値の文字の大きさ
    ## tcl: 軸の目盛線の長さ（＋は内側，－は外側に目盛を入れる）

  ## 右側の縦軸のラベル
  text.x <- x.max - ((x.max - x.min) / 1000)
    ## /の後ろの数字は、軸ラベルが右側の縦軸から左方向にどれだけ離すかを示す（通常は1000くらい？）
  text.y <- length(bpp_data.2) / 2
  text("試行数", x = text.x, y = text.y, col = "black", srt = 90, cex = 1) 
    ## cex: 文字の大きさ
}

##################################################
## 準備完了の表示
##################################################
cat("「bpp_stats_function.R」\n準備完了！\n")

##################################################
## 新たなデータの書き込み
##################################################
##  write.table(bpp_stats, file.name, fileEncoding="cp932")

##################################################
## FUJINO, Kazuhiro 2020
## http://fujino.la.coocan.jp/index.htm
##################################################
