##################################################
## BPP通算成績データの読み込み
##################################################
## 「bpp_stats_reader.R」で作成したデータの読み込み
bpp_stats <- read.table("bpp_stats1.txt", header = TRUE, fileEncoding="cp932", encoding="cp932")
## 「bpp_converter.R」で作成したデータの読み込み
bpp_data <- read.table("bpp_teamdata1.txt", header = TRUE, fileEncoding="cp932", encoding="cp932")

## HTMLの表にすると小数点以下2桁までしか表示されないので
## 勝率とチーム打率を1000倍しておく
bpp_data["WP"] <- 1000 * bpp_data["WP"]
bpp_data["AVG"] <- 1000 * bpp_data["AVG"]

##################################################
## 列名の設定（names()関数を使えば良いと思うけど、備忘用）
##################################################
bpp_data.list <- c("ID", "team", "W", "L", "D", 
                   "WP", "AVG", "ERA", 
                   "G", "AB", "H", "HR", "RBI", "SB", 
                   "BB", "SO", 
                   "GP", "IP", "CG", "S", "SP", 
                   "Ha", "HRa", "BBa", "K", "ER", 
                   "E", 
                   "option")

##################################################
## データフレームの内容を、HTMLの表を作成するタグ形式で出力
##################################################
html.table <- function(data) {
  library(xtable)
  ## 差し当たり必要な項目のみ表示
  data <- data[c(2:4, 7:27)]
  html.table <- xtable(data) 
  return(print(html.table, type = "html", include.rownames = FALSE))
}

##################################################
## 特定の「"option"」のデータのみ抽出
##################################################
option.data <- function(data.name, option.name) {
  option.data <- subset(data.name, data.name$option == option.name)
  invisible(option.data)
}

##################################################
## 特定の「"ID"」「"option"」の統計値（列）の要素のみを表示
##################################################
element <- function(data.name, id.nam, stats.name, option.name = "ALL") {
  if (option.name == "ALL") {
    i_data <- data.name[data.name$ID == id.nam, ] 
    ele <- i_data[stats.name]
   } else {
    i_data <- data.name[data.name$ID == id.nam & data.name$option == option.name, ] 
    ele <- i_data[stats.name]
  }
  return(ele)
}

##################################################
## 特定の「"ID"」「"option"」の統計値（列）の要素のみを表示
## （HTMLの表の作成用）
##################################################
ele <- function(data.name, id.nam, stats.name, option.name = "ALL") {
  if (option.name == "ALL") {
    i_data <- data.name[data.name$ID == id.nam, ] 
    element <- i_data[[stats.name]]
    ele <- paste("<td> ", element, " </td>", sep = "")
   } else {
    i_data <- data.name[data.name$ID == id.nam & data.name$option == option.name, ] 
    element <- i_data[[stats.name]]
    ele <- paste("<td bgcolor=#ffffcc> ", element, " </td>", sep = "")
  }
  return(ele)
}

##################################################
## 「bpp_converter.R」で作成したデータから
## 標本平均と標準誤差のデータを作成（HTMLの表の作成用）
##################################################
mean_and_sd <- function() {

  ## 標本平均のデータフレームと標準誤差のデータフレームを作成
  bpp.mean <- bpp_data.mean()
  bpp.sd <- bpp_data.sd()

  ## 結果を合成する列の抽出
  bpp.mean.2 <- bpp.mean[3:27]
  bpp.sd.2 <- bpp.sd[3:27]
  ## チーム名の列の抽出
  bpp.mean.1 <- bpp.mean[1:2]
  ## オプション名の列の抽出
  bpp.mean.3 <- bpp.mean[28]

  ## i行目のデータを抽出（データフレームの要素全体を一括操作できないので）
  mean.sd <- data.frame(NULL)  # 各行を結合するためのダミーデータフレーム
  for (i in 1:nrow(bpp.mean.2)) {

    ## i行目のデータを小数点以下2桁まで表示する
    i_bpp.mean.2 <- sprintf("%.2f", bpp.mean.2[i, ])
    i_bpp.sd.2 <- sprintf("%.2f", bpp.sd.2[i, ])

    ## i行目のデータを「標本平均<br>(標準誤差)」の形にデータを変換
    i_mean.sd <- paste(i_bpp.mean.2, "<br>(", i_bpp.sd.2, ")", sep = "")

    ## i行目のデータをデータフレームオブジェクトに変換
    i_mean.sd <- matrix(i_mean.sd, nrow = 1, ncol = 25)
    i_mean.sd <- data.frame(i_mean.sd)

　   ## (i-1)行目までのデータフレームオブジェクトと結合
    mean.sd <- rbind(mean.sd, i_mean.sd)

  }

  ## チーム名・オプション名の付加
  mean_and_sd <- cbind(bpp.mean.1, mean.sd, bpp.mean.3)
  ## 列名の付加
  names(mean_and_sd) <- bpp_data.list

  ## 出力結果
  invisible(mean_and_sd)

}

##################################################
## 「bpp_converter.R」で作成したデータを用いて
## 2つの「"option"」のデータの標本平均の差と
## 標本平均の差の標準誤差を計算（HTMLの表の作成用）
##################################################
#### なお、危険度1％と5％での有意差の有無も同時に表示しているので、
#### たとえば、テキストエディタの「編集→置換」で、
#### 「<td> TRUETRUE」のところを「<td bgcolor="#ff6699">」、
#### 「<td> FALSETRUE」のところを「<td bgcolor="#33ccff">」、
#### 「<td> FALSEFALSE」のところを「<td bgcolor="#ffffff">」、
#### ……のように置き換えれば、有意差のあるセルを色分けした表が作成できます。
##################################################
dif <- function(option1, option2) {

  ## 標本平均の差のデータフレームを作成
  bpp.dif <- dif.mean(option1, option2)
  ## 標本平均の差の標準誤差のデータフレームを作成
  bpp.dif_sd <- dif.mean.sd(option1, option2)
  ## 危険度1％と5％での有意差の有無を判定したデータフレームを作成
  bpp.1_sig <- bpp_sig(option1, option2, 0.01)
  bpp.5_sig <- bpp_sig(option1, option2, 0.05)

  ## 結果を合成する列の抽出
  bpp.dif.2 <- bpp.dif[3:27]
  bpp.dif_sd.2 <- bpp.dif_sd[3:27]
  bpp.1_sig.2 <- bpp.1_sig[3:27]
  bpp.5_sig.2 <- bpp.5_sig[3:27]
  ## チーム名の列の抽出
  bpp.dif.1 <- bpp.dif[1:2]
  ## オプション名の列の作成
  option_element <- paste(option1, "-", option2, sep = "")
  option_j <- rep(option_element, length = nrow(bpp.dif.2))
  bpp.dif.3 <- data.frame(option = option_j)

  ## i行目のデータを抽出（データフレームの要素全体を一括操作できないので）
  dif.sd <- data.frame(NULL)  # 各行を結合するためのダミーデータフレーム
  for (i in 1:nrow(bpp.dif.2)) {

    ## i行目のデータを小数点以下2桁まで表示する
    i_bpp.dif.2 <- sprintf("%.2f", bpp.dif.2[i, ])
    i_bpp.dif_sd.2 <- sprintf("%.2f", bpp.dif_sd.2[i, ])

    ## i行目のデータを
    ## 「危険度1％で有意・危険度5％で有意・標本平均の差<br>(標本平均の差の標準誤差)」
    ## の形にデータを変換
    i_dif.sd <- paste(
                      bpp.1_sig.2[i, ], 
                      bpp.5_sig.2[i, ], 
                      " ",
                      i_bpp.dif.2, 
                      "<br>(", i_bpp.dif_sd.2, ")", 
                      sep = ""
                      )

    ## i行目のデータをデータフレームオブジェクトに変換
    i_dif.sd <- matrix(i_dif.sd, nrow = 1, ncol = 25)
    i_dif.sd <- data.frame(i_dif.sd)

　   ## (i-1)行目までのデータフレームオブジェクトと結合
    dif.sd <- rbind(dif.sd, i_dif.sd)

  }

  ## チーム名・オプション名の付加
  dif <- cbind(bpp.dif.1, dif.sd, bpp.dif.3)
  ## 列名の付加
  names(dif) <- bpp_data.list

  ## 出力結果
  invisible(dif)

}

##################################################
## 「bpp_converter.R」で作成したデータから
## 「"option"」条件ごとに標本平均のデータを作成
##################################################
bpp_data.mean <- function() {

  ## チーム名とオプション名のリストの作成
  team.list <- unique(bpp_data$team)
  option.list <- unique(bpp_data$option)

  ## オプション条件ごとに標本平均を作成
  bpp_mean <- data.frame(NULL)
  for (i in option.list) {
    ## オプション条件に合うデータを抽出
    i_option.stats <- subset(bpp_data, bpp_data$option == i)
    ## 各チームごとに標本平均を計算
    j_option.mean <- data.frame(NULL)
    for (j in 1:12) {
      ## チームごとにデータを抽出
      j_team.stats <- subset(i_option.stats, i_option.stats$ID == j)
      ## 標本平均を計算する列の抽出
      j_team.stats <- j_team.stats[3:27]
      ## 標本平均の計算
      j_team.mean <- apply(j_team.stats, MARGIN = 2, mean)
      j_team.mean <- matrix(j_team.mean, nrow = 1, ncol = 25)
      j_team.mean <- data.frame(j_team.mean)
      ## 付加情報の追加
      j_team.mean <- cbind(j, team.list[j], j_team.mean, i)
      ## 標本平均データの結合
      j_option.mean <- rbind(j_option.mean, j_team.mean)
    }
    ## オプション条件ごとの標本平均データの結合
    bpp_mean <- rbind(bpp_mean, j_option.mean)
  }

  ## 列名の付加
  names(bpp_mean) <- bpp_data.list
  ## 出力結果
  invisible(bpp_mean)
}

##################################################
## 「bpp_converter.R」で作成したデータから
## 「"option"」条件ごとに標準誤差のデータを作成
##################################################
bpp_data.sd <- function() {

  ## チーム名とオプション名のリストの作成
  team.list <- unique(bpp_data$team)
  option.list <- unique(bpp_data$option)

  ## オプション条件ごとに標準誤差を作成
  bpp_sd <- data.frame(NULL)
  for (i in option.list) {
    ## オプション条件に合うデータを抽出
    i_option.stats <- subset(bpp_data, bpp_data$option == i)
    ## 各チームごとに標準誤差を計算
    j_option.sd <- data.frame(NULL)
    for (j in 1:12) {
      ## チームごとにデータを抽出
      j_team.stats <- subset(i_option.stats, i_option.stats$ID == j)
      ## 標準誤差を計算する列の抽出
      j_team.stats <- j_team.stats[3:27]
      ## 標準誤差の計算
      j_team.sd <- apply(j_team.stats, MARGIN = 2, sd)
      j_team.sd <- matrix(j_team.sd, nrow = 1, ncol = 25)
      j_team.sd <- data.frame(j_team.sd)
      ## 付加情報の追加
      j_team.sd <- cbind(j, team.list[j], j_team.sd, i)
      ## 標準誤差データの結合
      j_option.sd <- rbind(j_option.sd, j_team.sd)
    }
    ## オプション条件ごとの標準誤差データの結合
    bpp_sd <- rbind(bpp_sd, j_option.sd)
  }

  ## 列名の付加
  names(bpp_sd) <- bpp_data.list
  ## 出力結果
  invisible(bpp_sd)
}

##################################################
## 「bpp_converter.R」で作成したデータを用いて
## 2つの「"option"」のデータの標本平均の差を計算
##################################################
dif.mean <- function(option1, option2) {

 ## チーム名のリストの作成
  team.list <- unique(bpp_data$team)

  ## オプション条件に合うデータを抽出
  bpp_data1 <- subset(bpp_data, bpp_data$option == option1)
  bpp_data2 <- subset(bpp_data, bpp_data$option == option2)

  ## 各チームごとに標本平均の差を計算
  dif.mean <- data.frame(NULL)
  for (i in 1:12) {
    ## チームごとにデータを抽出
    i_team.stats1 <- subset(bpp_data1, bpp_data1$ID == i)
    i_team.stats2 <- subset(bpp_data2, bpp_data1$ID == i)
    ## 標本平均を計算する列の抽出
    i_team.stats1 <- i_team.stats1[3:27]
    i_team.stats2 <- i_team.stats2[3:27]
    ## 標本平均の計算
    i_team.mean1 <- apply(i_team.stats1, MARGIN = 2, mean)
    i_team.mean2 <- apply(i_team.stats2, MARGIN = 2, mean)
    ## 標本平均の差の計算
    i_team.mean <- i_team.mean1 - i_team.mean2
    i_team.mean <- matrix(i_team.mean, nrow = 1, ncol = 25)
    i_team.mean <- data.frame(i_team.mean)
    ## 付加情報の追加
    i_team.mean <- cbind(i, team.list[i], i_team.mean)
    ## 各チームごとの標本平均の差データの結合
    dif.mean <- rbind(dif.mean, i_team.mean)
  }

  ## 列名の付加
  names(dif.mean) <- bpp_data.list[1:27]
  ## 出力結果
  invisible(dif.mean)
}

##################################################
## 「bpp_converter.R」で作成したデータを用いて
## 2つの「"option"」のデータの標本平均の差の標準誤差を計算
##################################################
dif.mean.sd <- function(option1, option2) {

 ## チーム名のリストの作成
  team.list <- unique(bpp_data$team)

  ## オプション条件に合うデータを抽出
  bpp_data1 <- subset(bpp_data, bpp_data$option == option1)
  bpp_data2 <- subset(bpp_data, bpp_data$option == option2)

  ## 各チームごとに標本平均の差の標準誤差を計算
  dif.mean.sd <- data.frame(NULL)
  for (i in 1:12) {
    ## チームごとにデータを抽出
    i_team.stats1 <- subset(bpp_data1, bpp_data1$ID == i)
    i_team.stats2 <- subset(bpp_data2, bpp_data1$ID == i)
    ## 計算する列の抽出
    i_team.stats1 <- i_team.stats1[3:27]
    i_team.stats2 <- i_team.stats2[3:27]
    ## 標本平均の差の分散の計算
    i_team.var1 <- apply(i_team.stats1, MARGIN = 2, var)
    i_team.var2 <- apply(i_team.stats2, MARGIN = 2, var)
    ## 標本サイズの計算
    i_team.n1 <- apply(i_team.stats1, MARGIN = 2, length)
    i_team.n2 <- apply(i_team.stats2, MARGIN = 2, length)
    ## 標本平均の差の標準誤差の計算
    i_team.sd <- sqrt((i_team.var1 / i_team.n1) + (i_team.var2 / i_team.n2))
    i_team.sd <- matrix(i_team.sd, nrow = 1, ncol = 25)
    i_team.sd <- data.frame(i_team.sd)
    ## 付加情報の追加
    i_team.sd <- cbind(i, team.list[i], i_team.sd)
    ## 各チームごとの標本平均の差データの結合
    dif.mean.sd <- rbind(dif.mean.sd, i_team.sd)
  }

  ## 列名の付加
  names(dif.mean.sd) <- bpp_data.list[1:27]
  ## 出力結果
  invisible(dif.mean.sd)
}

##################################################
## 「bpp_converter.R」で作成したデータを用いて
## 2つの「"option"」のデータの差について有意差の有無を判定
##################################################
bpp_sig <- function(option1, option2, alpha = 0.05) {

  ## チーム名のリストの作成
  team.list <- unique(bpp_data$team)

  ## オプション条件に合うデータを抽出
  bpp_data1 <- subset(bpp_data, bpp_data$option == option1)
  bpp_data2 <- subset(bpp_data, bpp_data$option == option2)

  ## チームごとにp値を計算
  sig.table <- NULL
  for (i in 1:12) {
    ## チームごとにデータを抽出
    team.data1 <- subset(bpp_data1, bpp_data1$ID == i)
    team.data2 <- subset(bpp_data2, bpp_data2$ID == i)
    ## 列ごとにp値を計算し、有意差の有無を判定
    sig.vector <- NULL
    for (j in 3:27) {
      test.result <- t.test(team.data1[j], team.data2[j], var.equal = FALSE)
      p.value <- test.result$p.value
      ## 有意差の有無を判定
      sig.value <- p.value < alpha
      sig.vector <- c(sig.vector, sig.value) 
    }
    ## 型変換
    sig.matrix <- matrix(sig.vector, nrow = 1, ncol = 25)
    sig.dataframe <- data.frame(sig.matrix)
    ## 付加情報の追加
    sig.dataframe <- cbind(i, team.list[i], sig.dataframe)
    ## オプション条件ごとの有意差データの結合
    sig.table <- rbind(sig.table, sig.dataframe)
  }

  ## 列名の付加
  names(sig.table) <- bpp_data.list[1:27]
  ## 出力結果
  invisible(sig.table)
}

##################################################
## 2つのパラメータ（たとえば、守備力と肩）をそれぞれ変えながら、
## 結果を比較したHTMLの表を作成
##################################################
#### 「bpp_converter.R」で作成したデータから
#### 標本平均と標準誤差のデータ
#### および、
#### 2つの「"option"」のデータの標本平均の差と
#### 標本平均の差の標準誤差を計算
##################################################
#### なお、危険度1％と5％での有意差の有無も同時に表示しているので、
#### たとえば、テキストエディタの「編集→置換」で、
#### 「<td> TRUETRUE」のところを「<td bgcolor="#ff6699">」、
#### 「<td> FALSETRUE」のところを「<td bgcolor="#33ccff">」、
#### 「<td> FALSEFALSE」のところを「<td bgcolor="#ffffff">」、
#### ……のように置き換えれば、有意差のあるセルを色分けした表が作成できます。
##################################################
#### 標本平均のセルと空白のセルは、たとえば、テキストエディタの「編集→置換」で、
#### 「<td> MEAN」のところを「<td bgcolor="#ffffcc">」、
#### 「<td> NULL_CELL </td>」のところを「<td></td>」
#### として下さい。
##################################################
two_way_compare <- function(stats.name) {

  ## 初期設定
  team.id <- 9
  prefix <- "BW_SS_"

  ## パラメータのリスト
  suffix <- c("S", "A", "B", "C", "D", "E")
  ## 空白のセルの設定
  null_cell <- list("NULL_CELL")
  names(null_cell) <- stats.name
  null_cell <- data.frame(null_cell)

  ## 標本平均のデータフレームと標準誤差のデータフレームを作成
  bpp.mean <- bpp_data.mean()
  bpp.sd <- bpp_data.sd()

  ## 比較表の作成
  for (i in 1:6) {

    ## 第1変数が「i」のときの列データを作成
    col_element_1 <- data.frame(NULL)
    for (j in 1:6) {

      ## 第1変数「i」・第2変数「j」のデータの標本平均と標本誤差
      # オプション名の設定
      option_name <- paste(prefix, suffix[i], suffix[j], sep = "")
      # 第1変数「i」・第2変数「j」のデータの標本平均と標本誤差の抽出
      mean_element <- bpp.mean[bpp.mean$ID == team.id & bpp.mean$option == option_name, ]
      mean_element <- mean_element[[stats.name]]
      sd_element <- bpp.sd[bpp.sd$ID == team.id & bpp.sd$option == option_name, ]
      sd_element <- sd_element[[stats.name]]
      # 標本平均と標本誤差を小数点以下2桁まで表示する
      mean_element.2 <- sprintf("%.2f", mean_element)
      sd_element.2 <- sprintf("%.2f", sd_element)
      # 「MEAN 標本平均<br>(標準誤差)」の形にデータを変換
      mean_sd_element <- paste("MEAN ", mean_element.2, "<br>(", sd_element.2, ")", sep = "")
      # データをデータフレームオブジェクトに変換
      mean_sd_element <- list(mean_sd_element)
      names(mean_sd_element) <- stats.name
      mean_sd_element <- data.frame(mean_sd_element)
      col_element_1 <- rbind(col_element_1, mean_sd_element)

      ## 第1変数「i」・第2変数「j」のデータと、第1変数「i」・第2変数「j+1」のデータの、
      ## 標本平均の差と、その標準誤差
      if (j < 6) { 
         option1_name <- paste(prefix, suffix[i], suffix[j], sep = "")
         option2_name <- paste(prefix, suffix[i], suffix[j+1], sep = "")
        dif <- dif(option1_name, option2_name)
         option_name <- paste(option1_name, "-", option2_name, sep = "")
        dif_element <- element(dif, team.id, stats.name, option_name)
        col_element_1 <- rbind(col_element_1, dif_element)
      }
    }
    if (i == 1) {
     table_element <- col_element_1
    } else {
     table_element <- cbind(table_element, col_element_1)
    }

    ## 第1変数が「i」←「i+1」に変化したときの列データを作成
    if (i < 6) {
      col_element_2 <- data.frame(NULL)
      for (j in 1:6) {
        ## 第1変数「i」・第2変数「j」のデータと、第1変数「i+1」・第2変数「j」のデータの、
        ## 標本平均の差と、その標準誤差
         option1_name <- paste(prefix, suffix[i], suffix[j], sep = "")
         option2_name <- paste(prefix, suffix[i+1], suffix[j], sep = "")
        dif <- dif(option1_name, option2_name)
         option_name <- paste(option1_name, "-", option2_name, sep = "")
        dif_element <- element(dif, team.id, stats.name, option_name)
        col_element_2 <- rbind(col_element_2, dif_element)
        if (j < 6) {
          col_element_2 <- rbind(col_element_2, null_cell)
        }
      }
      table_element <- cbind(table_element, col_element_2)
    }
  }

  ## 出力結果
  library(xtable)
  html.table <- xtable(table_element) 
  return(print(html.table, type = "html", include.rownames = FALSE))
}

##################################################
## 準備完了の表示
##################################################
cat("「html出力用関数リスト」\n準備完了！\n")
