##################################################
## 初期設定
##################################################
## オプション名の設定
option.name <- "ORIGINAL"

## 試合数の設定
season.games <- 135

## チーム・個人成績データをセーブするファイルの名前の設定
file.name <- "bpp_stats1.txt"

## 個人成績データを記録するかどうかの選択
#### 野手の個人成績データを記録： person.flag <- "b"
#### 投手の個人成績データを記録： person.flag <- "p"
#### 個人成績データは記録しない： person.flag <- "non"
person.flag <- "non"

## 個人成績データを記録する選手の所属チームと名前の設定
person.team <- "Ｄ"
person.name <- "関川"

##################################################
## 個人打撃成績データの作成
##################################################
## 野手成績リスト「b_stats.txt」の読み込み
b.stats_all <- read.table("b_stats.txt", skip = 1, fileEncoding="cp932", encoding="cp932")

## 日本シリーズまで完了しているかの確認（その1）
nrow_b.stats_all <- nrow(b.stats_all)
if (nrow_b.stats_all != 392) {
  cat("\n■日本シリーズが終了していない野手成績リストです。\n")
  cat("■日本シリーズまで終了させた野手成績リストを用意して下さい。\n\n")
}
stopifnot(nrow_b.stats_all == 392)

## 日本シリーズまで完了しているかの確認（その2）
b.nihon.series <- tail(b.stats_all, n = 56)
sum_b.nihon.series <- sum(b.nihon.series[[4]])
if (sum_b.nihon.series == 0) {
  cat("\n■日本シリーズが終了していない野手成績リストです。\n")
  cat("■日本シリーズまで終了させた野手成績リストを用意して下さい。\n\n")
}
stopifnot(sum_b.nihon.series != 0)

## ペナントレースの成績データだけを抽出
b.stats <- head(b.stats_all, n = 336)

## 試合数列とオプション列を付加
s.games <- rep(season.games, 336)
b.option <- rep(option.name, 336)
b.stats <- cbind(b.stats, s.games, b.option)

## b.statsにラベルを付ける
label_b.stats <- c("b.name", "b.team", "b.avg", "b.games", "b.ab", 
                   "b.hits", "b.2b", "b.3b", "b.hr", "b.rbi", "b.sh", 
                   "b.bb", "b.so", "b.sb", "b.errors", "b.slg", "b.obp", 
                   "b.s_games", "b.option")
names(b.stats) <- label_b.stats

## チーム記号の重複の有無の確認
b.stats_teams <- unique(b.stats$b.team)
b.stats_teams.num <- length(b.stats_teams)
if (b.stats_teams.num != 12) {
  cat("\n■チーム記号に重複があります。\n■『ベスプレ』のチームデータを確認して下さい。\n\n")
}
stopifnot(b.stats_teams.num == 12)

##################################################
## 個人投手成績データの作成
##################################################
## 日本シリーズまで完了しているかの確認（その1）
p.stats.flag <- 0
read_p.stats <- function() {
  bpp_p.stats <- read.table("p_stats.txt", skip = 168, nrow = 1, fileEncoding="cp932", encoding="cp932")
}
read_p.stats.flag <- try(read_p.stats(), silent = FALSE)
  if (class(read_p.stats.flag) == "try-error") {
  cat("\n■日本シリーズが終了していない投手成績リストです。\n")
  cat("■日本シリーズまで終了させた投手成績リストを用意して下さい。\n\n")
  p.stats.flag <- 1
  }
stopifnot(p.stats.flag == 0)

## p.statsのラベルの定義
label_p.stats <- c("p.name", "p.team", "p.era", "p.games", "p.cg", 
                   "p.wins", "p.losses", "p.saves", "p.sp", "p.ip",  
                   "p.hits", "p.hr", "p.bb", "p.so", "p.er", "p.wpct", "p.kper9", 
                   "p.s_games", "p.option")

## 投手成績リスト「p_stats.txt」の読み込み
p.stats_all <- NULL
for (i in 1:168) {
  i_p.stats_all <- read.table("p_stats.txt", skip = i, nrow = 1, fileEncoding="cp932", encoding="cp932")
  if (ncol(i_p.stats_all) == 17) {
    ## 投球イニング数の半端がない場合
    ## 試合数列とオプション列を付加
    i_p.stats_all <- cbind(i_p.stats_all, season.games, option.name)
    ## i_p.stats_allにラベルを付ける
    names(i_p.stats_all) <- label_p.stats
    ## p.stats_allに新たなi_p.stats_allを付加
    p.stats_all <- rbind(p.stats_all, i_p.stats_all)

  } else if (i_p.stats_all[11] == "1/3") {
    ## 投球イニング数の半端が1/3の場合
    i_p.stats_all[10] <- i_p.stats_all[10] + (1 / 3)
    i_p.stats_all <- i_p.stats_all[c(1:10, 12:18)]
    ## 試合数列とオプション列を付加
    i_p.stats_all <- cbind(i_p.stats_all, season.games, option.name)
    ## i_p.stats_allにラベルを付ける
    names(i_p.stats_all) <- label_p.stats
    ## p.stats_allに新たなi_p.stats_allを付加
    p.stats_all <- rbind(p.stats_all, i_p.stats_all)

  } else if (i_p.stats_all[11] == "2/3") {
    ## 投球イニング数の半端が2/3の場合
    i_p.stats_all[10] <- i_p.stats_all[10] + (2 / 3)
    i_p.stats_all <- i_p.stats_all[c(1:10, 12:18)]
    ## 試合数列とオプション列を付加
    i_p.stats_all <- cbind(i_p.stats_all, season.games, option.name)
    ## i_p.stats_allにラベルを付ける
    names(i_p.stats_all) <- label_p.stats
    ## p.stats_allに新たなi_p.stats_allを付加
    p.stats_all <- rbind(p.stats_all, i_p.stats_all)
 }
}

## 日本シリーズまで完了しているかの確認（その2）
p.nihon.series <- tail(p.stats_all, n = 24)
sum_p.nihon.series <- sum(p.nihon.series[[4]])
if (sum_p.nihon.series == 0) {
  cat("\n■日本シリーズが終了していない投手成績リストです。\n")
  cat("■日本シリーズまで終了させた投手成績リストを用意して下さい。\n\n")
}
stopifnot(sum_p.nihon.series != 0)

## ペナントレースの成績データだけを抽出
p.stats <- head(p.stats_all, n = 144)

## チーム記号の重複の有無の確認
p.stats_teams <- unique(p.stats$p.team)
p.stats_teams.num <- length(p.stats_teams)
if (p.stats_teams.num != 12) {
    cat("\n■チーム記号に重複があります。\n■『ベスプレ』のチームデータを確認して下さい。\n\n")
}
stopifnot(p.stats_teams.num == 12)

##################################################
## 野手成績リストと野手成績リストのチーム記号が一致しているかの確認
##################################################
chara_b.stats_teams <- as.character(b.stats_teams)
chara_p.stats_teams <- as.character(p.stats_teams)
b.p._teames_agree <- chara_b.stats_teams == chara_p.stats_teams
b.p._teames_agree_num <- length(b.p._teames_agree[b.p._teames_agree == TRUE])
if (b.p._teames_agree_num != 12) {
  cat("\n■野手成績リストと野手成績リストのチーム記号が一致していません。\n\n")
}
stopifnot(b.p._teames_agree_num == 12)

##################################################
## 個人成績データの作成
##################################################
## 野手の場合
if (person.flag == "b") {
  person.stats <- b.stats[b.stats$b.name == person.name & b.stats$b.team == person.team, ]
  if (nrow(person.stats) == 0) {
    cat("\n■個人成績データを記録する選手として設定した該当選手がいません。")
    cat("\n■スクリプトを開いて、初期設定を確認して下さい。\n\n")
  }
  if (nrow(person.stats) >= 2) {
    cat("\n■個人成績データを記録する選手として設定した該当選手と\n同名の選手が2人以上います。")
    cat("\n■『ベスプレ』の選手データを確認して下さい。\n\n")
  }
  stopifnot(nrow(person.stats) == 1)
  person.stats <- data.frame("team" = 0, 
                             "name" = paste(person.flag, ":", person.team, person.name, sep = "" ), 
                             "b.games" = person.stats[[1, "b.games"]], 
                             "b.ab" = person.stats[[1, "b.ab"]], 
                             "b.hits" = person.stats[[1, "b.hits"]], 
                             "b.2b" = person.stats[[1, "b.2b"]], "b.3b" = person.stats[[1, "b.3b"]], 
                             "b.hr" = person.stats[[1, "b.hr"]], 
                             "b.rbi" = person.stats[[1, "b.rbi"]], 
                             "b.sh" = person.stats[[1, "b.sh"]], "b.bb" = person.stats[[1, "b.bb"]], 
                             "b.so" = person.stats[[1, "b.so"]], "b.sb" = person.stats[[1, "b.sb"]], 
                             "b.errors" = person.stats[[1, "b.errors"]], 
                             "p.games" = NA, "p.cg" = NA, "p.wins" = NA, "p.losses" = NA, 
                             "p.saves" = NA, "p.sp" = NA, "p.ip" = NA, 
                             "p.hits" = NA, "p.hr" = NA, "p.bb" = NA, "p.so" = NA, "p.er" = NA, 
                             "s.games" = season.games, "option" = option.name)
}

## 投手の場合
if (person.flag == "p") {
  person.stats <- p.stats[p.stats$p.name == person.name & p.stats$p.team == person.team, ]
  if (nrow(person.stats) == 0) {
    cat("\n■個人成績データを記録する選手として設定した該当選手がいません。")
    cat("\n■スクリプトを開いて、初期設定を確認して下さい。\n\n")
  }
  if (nrow(person.stats) >= 2) {
    cat("\n■個人成績データを記録する選手として設定した該当選手と\n同名の選手が2人以上います。")
    cat("\n■『ベスプレ』の選手データを確認して下さい。\n\n")
  }
  stopifnot(nrow(person.stats) == 1)
  person.stats <- data.frame("team" = 0, 
                             "name" = paste(person.flag, ":", person.team, person.name, sep = "" ), 
                             "b.games" = NA, "b.ab" = NA, "b.hits" = NA, 
                             "b.2b" = NA, "b.3b" = NA, "b.hr" = NA, "b.rbi" = NA, 
                             "b.sh" = NA, "b.bb" = NA, "b.so" = NA, "b.sb" = NA, "b.errors" = NA, 
                             "p.games" = person.stats[[1, "p.games"]], 
                             "p.cg" = person.stats[[1, "p.cg"]], 
                             "p.wins" = person.stats[[1, "p.wins"]], 
                             "p.losses" = person.stats[[1, "p.losses"]], 
                             "p.saves" = person.stats[[1, "p.saves"]], 
                             "p.sp" = person.stats[[1, "p.sp"]], "p.ip" = person.stats[[1, "p.ip"]], 
                             "p.hits" = person.stats[[1, "p.hits"]], 
                             "p.hr" = person.stats[[1, "p.hr"]], 
                             "p.bb" = person.stats[[1, "p.bb"]], "p.so" = person.stats[[1, "p.so"]], 
                             "p.er" = person.stats[[1, "p.er"]], 
                             "s.games" = season.games, "option" = option.name)
}

## 上記以外の場合
if ((person.flag != "b") & (person.flag != "p")) {
  person.stats <- data.frame("team" = 0, "name" = "norecord", 
                             "b.games" = NA, "b.ab" = NA, "b.hits" = NA, 
                             "b.2b" = NA, "b.3b" = NA, "b.hr" = NA, "b.rbi" = NA, 
                             "b.sh" = NA, "b.bb" = NA, "b.so" = NA, "b.sb" = NA, "b.errors" = NA, 
                             "p.games" = NA, "p.cg" = NA, "p.wins" = NA, 
                             "p.losses" = NA, "p.saves" = NA, "p.sp" = NA, "p.ip" = NA, 
                             "p.hits" = NA, "p.hr" = NA, "p.bb" = NA, "p.so" = NA, "p.er" = NA, 
                             "s.games" = season.games, "option" = option.name)
}

##################################################
## チーム・個人成績データの作成
##################################################
## チーム名のリストを作成
team.list <- unique(b.stats$b.team)

## チーム・個人成績データの作成
team.stats <- person.stats
for (i in 1:12) {
  ## チームごとに個人成績データを抽出
  i_b.stats <- b.stats[b.stats$b.team == team.list[i], ]
  i_p.stats <- p.stats[p.stats$p.team == team.list[i], ]
  ## チームごとにチーム成績データを作成
  i_stats <- data.frame("team" = i, "name" = team.list[i], 
                        "b.games" = sum(i_b.stats$b.games), 
                        "b.ab" = sum(i_b.stats$b.ab), "b.hits" = sum(i_b.stats$b.hits), 
                        "b.2b" = sum(i_b.stats$b.2b), "b.3b" = sum(i_b.stats$b.3b), 
                        "b.hr" = sum(i_b.stats$b.hr), "b.rbi" = sum(i_b.stats$b.rbi), 
                        "b.sh" = sum(i_b.stats$b.sh), "b.bb" = sum(i_b.stats$b.bb), 
                        "b.so" = sum(i_b.stats$b.so), "b.sb" = sum(i_b.stats$b.sb), 
                        "b.errors" = sum(i_b.stats$b.errors), 
                        "p.games" = sum(i_p.stats$p.games), 
                        "p.cg" = sum(i_p.stats$p.cg), "p.wins" = sum(i_p.stats$p.wins), 
                        "p.losses" = sum(i_p.stats$p.losses), "p.saves" = sum(i_p.stats$p.saves), 
                        "p.sp" = sum(i_p.stats$p.sp), "p.ip" = sum(i_p.stats$p.ip), 
                        "p.hits" = sum(i_p.stats$p.hits), "p.hr" = sum(i_p.stats$p.hr), 
                        "p.bb" = sum(i_p.stats$p.bb), "p.so" = sum(i_p.stats$p.so), 
                        "p.er" = sum(i_p.stats$p.er), 
                        "s.games" = season.games, "option" = option.name)
  ## チーム・個人成績データの結合
  team.stats <- rbind(team.stats, i_stats)
}

##################################################
## 既存データの有無の確認と、
## チーム・個人成績データの読み込み、ないし、新規作成
##################################################
read_bpp_stats <- function() {
  bpp_stats <- read.table(file.name, header = TRUE, fileEncoding="cp932", encoding="cp932")
}
read_bpp_stats.flag <- try(read_bpp_stats(), silent = FALSE)
  if (class(read_bpp_stats.flag) == "try-error") {
    ## 既存データがない場合の処理
    cat(paste("\n■初期設定されたファイル名「", file.name, "」で、\n", sep = ""))
    cat("　新規にチーム・個人成績データファイルを作成します。\n")
    bpp_stats <- team.stats
    new_bpp_stats.flag <- 1
  } else {
    ## 既存データの読み込み
    bpp_stats <- read.table(file.name, header = TRUE, fileEncoding="cp932", encoding="cp932")
    new_bpp_stats.flag <- 0
  }

##################################################
##  「b_stats.txt」や「p_stats.txt」を更新せずに、
##  「bpp_stats_reader.R」を回してしまうミスなどを確認
##################################################
if (new_bpp_stats.flag == 0) {
  ## 既存データと新規データの最終行を抽出
  last.row_bpp_stats <- tail(bpp_stats, n = 1)
  last.row_team.stats <- tail(team.stats, n = 1)

  ## 野手データのみ抽出
  b_last.row_bpp_stats <- last.row_bpp_stats[c(2, 3:14)]
  b_last.row_team.stats <- last.row_team.stats[c(2, 3:14)]
  ## 投手データのみ抽出
  p_last.row_bpp_stats <- last.row_bpp_stats[c(2, 15:20, 22:26)]
  p_last.row_team.stats <- last.row_team.stats[c(2, 15:20, 22:26)]

  ## 野手データの重複の確認
  b_last.row_agree <- b_last.row_bpp_stats == b_last.row_team.stats
  b_last.row_agree.flag <- all(b_last.row_agree == TRUE)
  if (b_last.row_agree.flag == TRUE) {
    cat("■前回記録した野手成績リストと、同じ野手成績リストです。\n")
    cat("■新しい野手成績リスト「b_stats.txt」を用意して下さい。\n\n")
  }
  ## 投手データの重複の確認
  p_last.row_agree <- p_last.row_bpp_stats == p_last.row_team.stats
  p_last.row_agree.flag <- all(p_last.row_agree == TRUE)
  if (p_last.row_agree.flag == TRUE) {
    cat("■前回記録した投手成績リストと、同じ投手成績リストです。\n")
    cat("■新しい投手成績リスト「p_stats.txt」を用意して下さい。\n\n")
  }
  stopifnot(b_last.row_agree.flag != TRUE)
  stopifnot(p_last.row_agree.flag != TRUE)
}

##################################################
## 新シーズンのチーム・個人成績データを追加統合
##################################################
if (new_bpp_stats.flag == 0) {
  bpp_stats <- rbind(bpp_stats, team.stats)
}

##################################################
## 選手名・チーム名が一致しているかどうかのチェック
##################################################
team.name.num <- length(unique(bpp_stats$name))
if (team.name.num != 13) {
  team.name.list_all <- as.character(unique(bpp_stats$name))
  team.name.list_0 <- team.name.list_all[1:13]
  team.name.list_1 <- as.character(unique(team.stats$name))
  cat("■既存のチーム・個人成績データファイルと、異なる選手名・チーム名が使われています！\n\n")
  cat("既存のチーム・個人成績データファイルの選手名・チーム名：\n")
  print(team.name.list_0)
  cat("今回の試行での選手名・チーム名：\n")
  print(team.name.list_1)
  cat("\nひとつのチーム・個人成績データファイルに含まれる個人名は1人分、\nかつ、チーム名は必ず12種類でなければなりません。")
}
stopifnot(team.name.num == 13)

##################################################
## 試行回数の表示
##################################################
all_bpp_stats.size <- nrow(bpp_stats) / 13
option_bpp_stats.size <- nrow(subset(bpp_stats, bpp_stats$option == option.name)) / 13
cat(paste("■総試行回数： ", all_bpp_stats.size, "\n", sep = ""))
cat(paste("■今回のオプション名「", option.name, "」での試行回数： ", option_bpp_stats.size, "\n\n", sep = ""))

##################################################
## 新たなBPP通算チーム・個人成績データの書き込み
##################################################
write.table(bpp_stats, file.name, fileEncoding="cp932")

##################################################
## FUJINO, Kazuhiro 2020
## http://fujino.la.coocan.jp/index.htm
##################################################
